-----------------------
-------VARIABLES-------
-----------------------


local ADDON_NAME, namespace = ...;
local variables = namespace.variables;
local functions = namespace.functions;


-----------------------
-----SLASH FUNCTION----
-----------------------


SLASH_LLR1 = "/llr";
SLASH_LLR2 = "/lptlootroll";
SLASH_LLR3 = "/lpt";

SlashCmdList.LLR = 
function(msg)
	msg = strlower(msg);

	if variables.commandArray[msg] then
		variables.commandArray[msg].func();
		return;
	end
	
	print("");
	functions.printOut(GetAddOnMetadata("LPT Loot Roll", "Title") .. " version: " .. GetAddOnMetadata("LPT Loot Roll", "Version"));
	
	if msg == nil or msg == "" then  
		functions.printOut("Error: No command recieved.");
	else
		functions.printOut("Error: Command not recognized.");
	end

	functions.printCommands();
end


-----------------------
----FILTER FUNCTION----
-----------------------


--Filter function to replace the normal string with a link that promts the clicker if they wish to change their roll settings to the values of the sender.
--As seen in the weakaura addon (https://github.com/WeakAuras/WeakAuras2/blob/master/WeakAuras/Transmission.lua), simplified for my usage.
local function filterFunction(_, _, msg, ...)
	local _, _, name, rollValues = msg:find("%[LPTLootRoll: ([^%s]+) %- ([^%]]+)%]");

	if name and rollValues then
		msg = "|Hgarrmission:lptlootroll:rollvalues&" .. rollValues .. "|h|cFF00CCFF[LPT Roll Settings: " .. name .. "]|h|r";
		return false, msg, ...;
	end
end

ChatFrame_AddMessageEventFilter("CHAT_MSG_SAY", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_YELL", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY_LEADER", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_LEADER", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_INSTANCE_CHAT", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_INSTANCE_CHAT_LEADER", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_GUILD", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_OFFICER", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_WHISPER", filterFunction);
ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_WHISPER_INFORM", filterFunction);

--Get data from the custom hyperlinks.
hooksecurefunc("SetItemRef", 
function(link, text)
	local linkType, addon, param1 = strsplit(":", link);
	
	if linkType == "garrmission" and addon == "lptlootroll" then
		local func, value = strsplit("&", param1);

		if func and func ~= "" and value and value ~= ""  then
			if func == "rollvalues" then 
				local dialog = StaticPopup_Show("LPTLootRoll_SettingShare");

				if dialog then
					dialog.data = value;
				end
			elseif func == "command" then
				if variables.commandArray[value] then
					variables.commandArray[value].func();
				end
			elseif func == "roller" then
				if not variables.rollHistory[1].distributed then 
					local dialog = StaticPopup_Show("LPTLootRoll_DistributeItem", value);
					
					if dialog then
						dialog.data = value;
					end
				end
			end
		end
	end
end);


-----------------------
----GLOBAL FUNCTIONS---
-----------------------


--Function for printing out all available commands
function functions.printCommands()
	print("");
	functions.printOut("Available commands are:");
	for i, v in pairs(variables.commandArray) do
		print("|Hgarrmission:lptlootroll:command&" .. i .. "|h|cFF00CCFF[" ..i:gsub("^%l", strupper) .. "]|h|r|cFFFFFF00 - " .. v.description);
	end
end

--Custom printout method for yellow message printouts.
function functions.printOut(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg,1,1,0);
end

--Input the current item into the roll frame.
function functions.setItemButtonData(frame, link)
	frame:SetScript("OnMouseDown", 
	function(self,button) 
		if IsModifiedClick("CHATLINK") then 
			ChatEdit_InsertLink(link);
		elseif IsModifiedClick("DRESSUP") then 
			DressUpItemLink(link);
		elseif IsShiftKeyDown() and button=="RightButton" then 
			OpenAzeriteEmpoweredItemUIFromLink(link) 
		end 
	end);
	
	frame:SetNormalTexture(select(10,GetItemInfo(link)));
	
	frame:SetScript("OnEnter", 
	function(self) 
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR"); 
		GameTooltip:ClearLines(); 
		GameTooltip:SetHyperlink(link); 
		GameTooltip:Show();
	end);

	frame:SetScript("OnLeave", 
	function(self) 
		GameTooltip:Hide();
		ResetCursor();
	end);
end

--Button factory to be used by both the leader and user window.
function functions.startButton(button, frame, firstAnchor, secondAnchor, xValue, yValue, title, xLength, yLength, func)
	button:SetPoint(firstAnchor, frame, secondAnchor, xValue, yValue);
	button:SetSize(xLength,yLength);
	button:SetText(title);
	button:SetScript("OnClick", func);
end

--Function for clearing a frame.
function functions.clearFrame(frame)
	--Clear list views to prepare for new item.
	frame:Clear();
	
	--Reset scrollbar values. 
	frame:GetParent().scrollBar:SetMinMaxValues(0,0);
	frame:GetParent().scrollBar:SetValue(select(1, frame:GetParent().scrollBar:GetMinMaxValues()));
end

--Function for controlling length of history array.
function functions.historyLengthControl()
	while #variables.rollHistory >= llrSettings.historyLength do
		tremove(variables.rollHistory);
	end
end

--Get raidleader
function functions.getRaidLeader()
	if UnitInRaid("player") then
		for i = 1, GetNumGroupMembers() do
			local name, rank = GetRaidRosterInfo(i);

			if name and rank == 2 then
				return name;
			end
		end
	end
end

--Function for handling manual rolls.
function functions.manualRoll(self)
	local number = tonumber(self.editBox:GetText());

	if number then
		if functions.isUserShown() then
			functions.hideUser();
		end
		
		if functions.isLeaderShown() then
			functions.disableLeader();
		end
		
		RandomRoll(1,number);
	end

	self:Hide()
end

--Function for calculating scaling value. When scaling for screens with a higher Y resolution than 1200, you have to take parentscale into the account for it to scale properly.
function functions.calculateScale()
	local yResolution = tonumber(strmatch(GetCVar("gxWindowedResolution"), "%d+x(%d+)"));
	local parentScale = UIParent:GetScale();

	return 768/yResolution/(yResolution > 1200 and parentScale or 1);
end

--Function for showing the config window.
function functions.toggleConfig()
	InterfaceOptionsFrame_OpenToCategory(LPTLootRoll_Config);
	if not LPTLootRoll_Config:IsShown() then
		InterfaceOptionsFrame_OpenToCategory(LPTLootRoll_Config);
	end
end

--Function for hiding tooltip on mouseover.
function functions.hideGametoolTip()
	GameTooltip:Hide();
	ResetCursor();
end


-----------------------
------UI FACTORIES-----
-----------------------


--List factory.
function functions.scrollFrameFactory(frame, config, xValue, yValue, title, width, height, withBar)
	frame.width  = width;
	frame.height = height;
	frame:SetSize(frame.width, frame.height);
	frame:SetPoint("LEFT", config, "LEFT", xValue, yValue);
	frame:EnableMouse(true);
	frame:EnableMouseWheel(true);
	
	frame.title = frame:CreateFontString(nil,"OVERLAY");
	frame.title:SetFontObject("GameFontHighLight");
	frame.title:SetPoint("TOP", frame, "TOP", 0, 1);
	frame.title:SetText(title);
	
	--ScrollingMessageFrame
	local messageFrame = CreateFrame("ScrollingMessageFrame", nil, frame);
	messageFrame:SetPoint("TOPLEFT", 0, -17);
	messageFrame:SetPoint("BOTTOMRIGHT", 0, 38);
	messageFrame:SetFontObject("GameFontNormalSmall");
	messageFrame:SetTextColor(1, 1, 1, 1);
	messageFrame:SetJustifyH("LEFT");
	messageFrame:SetHyperlinksEnabled(true);
	messageFrame:SetFading(false);
	messageFrame:SetMaxLines(300);
	frame.messageFrame = messageFrame;
	
	local tex = messageFrame:CreateTexture(nil,"BACKGROUND");
	tex:SetAllPoints();
	tex:SetColorTexture(0, 0, 0);
	
	local backDrop = CreateFrame("Frame", nil, messageFrame);
	backDrop:SetPoint("TOPLEFT", -5, 6);
	backDrop:SetPoint("BOTTOMRIGHT", 2, -6);

	backDrop:SetBackdrop( 
		{ 
			bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
			edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
			tile = true, 
			tileSize = 16, 
			edgeSize = 16, 
			insets = 
			{ 
				left = 4, 
				right = 4, 
				top = 4, 
				bottom = 4 
			}
		}
	);
	backDrop:SetBackdropColor(0,0,0,0);
	
	-- Scroll bar
	if withBar then 
		local scrollBar = CreateFrame("Slider", nil, frame, "UIPanelScrollBarTemplate");
		scrollBar:SetPoint("RIGHT", frame, "RIGHT", 4, 11);
		scrollBar:SetSize(30, frame.height - 85);
		scrollBar:SetMinMaxValues(0, 0);
		scrollBar:SetValueStep(1);
		scrollBar:SetObeyStepOnDrag(true);
		scrollBar.scrollStep = 1;
		frame.scrollBar = scrollBar;

		scrollBar:SetScript("OnValueChanged", 
		function(self, value)
			messageFrame:SetScrollOffset(select(2, scrollBar:GetMinMaxValues()) - value);
		end);
		
		scrollBar:SetValue(0);

		frame:SetScript("OnMouseWheel", 
		function(self, delta)
			local cur_val = scrollBar:GetValue();
			local min_val, max_val = scrollBar:GetMinMaxValues();

			if delta < 0 and cur_val < max_val then
				cur_val = min(max_val, cur_val + 1);
				scrollBar:SetValue(cur_val);
			elseif delta > 0 and cur_val > min_val then
				cur_val = max(min_val, cur_val - 1);
				scrollBar:SetValue(cur_val);
			end
		end);
	end
end

--Check button factory.
function functions.checkButtonFactory(name, parent, x_loc, y_loc, displayname, tooltip)
	local checkbutton = CreateFrame("CheckButton", name, parent, "ChatConfigCheckButtonTemplate");
	checkbutton:SetPoint("TOPLEFT", x_loc, y_loc);
	_G[checkbutton:GetName().."Text"]:SetText(displayname);
	checkbutton.tooltip = tooltip;

	return checkbutton;
end

--Function for creating the roll setting frames.
function functions.editBoxFactory(frame, frameEditbox, parent, x, y, title, linkedFrame)
	frame:SetPoint("BOTTOMLEFT", parent, "BOTTOMLEFT", x, y);
	frame:SetSize(100,200);
	
	--Create and set title of the setting.
	frame.title = frame:CreateFontString(nil,"OVERLAY");
	frame.title:SetFontObject("GameFontHighLight");
	frame.title:SetPoint("CENTER", frame, "CENTER", 0, 5);
	frame.title:SetText(title);

	--Create the editbox for the setting.
	frame.editBox = frameEditbox;
	frame.editBox:SetPoint("CENTER", frame, "CENTER", 0, -15);
	frame.editBox:SetSize(40,40);
	frame.editBox:SetAutoFocus(false);
	frame.editBox:SetNumeric(true);
	frame.editBox:SetMaxLetters(4);
	frame.editBox:SetFontObject("GameFontHighLight");
	frame.editBox:SetScript("OnKeyDown", 
	function (self, key)
		if key == "TAB" then
			linkedFrame.editBox:SetFocus();
		end
	end);
end

--Function for creating the sliders.
function functions.sliderFactory(frame, frameSlider, x, y, title)
	frameSlider.tooltipText = "Sets scale (1 is default).";
	frameSlider:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", x, y);
	frameSlider:SetMinMaxValues(0.5, 2);
	frameSlider:SetWidth(120);
	frameSlider:SetOrientation("HORIZONTAL");
	frameSlider.stepValue = 0.1;
	frameSlider:SetValueStep(frameSlider.stepValue);
	
	_G[frameSlider:GetName().."Low"]:SetText("0.5");
	_G[frameSlider:GetName().."High"]:SetText("2");
	
	frameSlider.title = frameSlider:CreateFontString(nil, "OVERLAY");
	frameSlider.title:SetFontObject("GameFontHighLight");
	frameSlider.title:SetPoint("CENTER", frameSlider, "CENTER", 0, 30);
	frameSlider.title:SetText(title);
end