-----------------------
-------VARIABLES-------
-----------------------


local ADDON_NAME, namespace = ...;
local variables = namespace.variables;
local functions = namespace.functions;


-----------------------
-----POPUP WINDOWS-----
-----------------------


--Pop up for manual rolls.
StaticPopupDialogs["LPTLootRoll_ManualRoll"] = 
{
	text = "Enter roll (Max: 9999):",
	button1 = "Accept",
	button2 = "Cancel",
	hasEditBox = 1,
	maxLetters = 4,
	OnShow = function(self)
		local editBox = self.editBox;
		editBox:SetText("");
		editBox:SetFocus();
	end,
	OnAccept = function(self)
		functions.manualRoll(self);
	end,
	EditBoxOnEnterPressed = function(self)
		functions.manualRoll(self:GetParent());
	end,
	EditBoxOnEscapePressed = function(self)
		self:GetParent():Hide();
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};

--Pop up for resetting history.
StaticPopupDialogs["LPTLootRoll_ClearHistory"] = 
{
	text = "Are you sure you want to reset roll history?",
	button1 = "Accept",
	button2 = "Cancel",
	OnAccept = function(self)
		functions.resetRolls();
		functions.printOut("LLR: Item history has been reset.");
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};

--Pop up for resetting locations.
StaticPopupDialogs["LPTLootRoll_ResetPosition"] = 
{
	text = "Are you sure you want to reset window positions?",
	button1 = "Accept",
	button2 = "Cancel",
	OnAccept = function(self)
		functions.resetUserWindow();
        functions.resetLeaderWindow();
        functions.resetItemListWindow();
		functions.printOut("LLR: Location of windows have been reset.");
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};

--Pop up for sending unwanted loot.
StaticPopupDialogs["LPTLootRoll_SendTradeable"] =
{
	text = "Do you want to list this item for trade?",
	button1 = "Yes",
	button2 = "No",
	OnAccept = function(self, data)
		local _, link = GetItemInfo(data);

		variables.raidLeader = UnitIsGroupLeader(variables.raidLeader or "") and variables.raidLeader or functions.getRaidLeader();

		if variables.raidLeader then
			SendChatMessage(link, "WHISPER", nil, variables.raidLeader);
		end
	end,
	timeout = 0,
	whileDead = 1,
	noCancelOnReuse = 1,
	hasItemFrame = 1
};

--Pop up for accepting settings rolls.
StaticPopupDialogs["LPTLootRoll_SettingShare"] = {
	text = "Are you sure you want to change roll values?",
	button1 = "Accept",
	button2 = "Cancel",
	OnAccept = function(self, message)
		--If user accepts the data, parse it to numbers and store it in settings.
		local main, off, mog = strsplit("-", message);

		main = tonumber(main);
		off  = tonumber(off);
		mog  = tonumber(mog);
		
		if main and off and mog then
			llrSettings.mainRoll = main;
			llrSettings.offRoll = off;
			llrSettings.mogRoll = mog;
		end
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};

--Pop up for distributing loot.
StaticPopupDialogs["LPTLootRoll_DistributeItem"] =
{
	text = "Give item to %s?",
	button1 = "Accept",
	button2 = "Cancel",
	OnAccept = function(self, name)
		--If user accepts, tell who wins it in raidwarning.
		SendChatMessage("Loot goes to: " .. name, "RAID_WARNING", nil, nil);
		variables.rollHistory[1].distributed = true;
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};