-----------------------
----LOCAL VARIABLES----
-----------------------


local ADDON_NAME, namespace = ...
local functions = namespace.functions;
local variables = namespace.variables;

local itemList = CreateFrame("Frame", "LPTLootRoll_ItemList", nil, "BasicFrameTemplate");
local loadFrame = CreateFrame("FRAME");
local itemsToRoll = {};


-----------------------
----LOCAL FUNCTIONS----
-----------------------


--Function for controlling if the buttons should be active.
local function buttonControl()
    itemList.skipItemButton:SetEnabled(#itemsToRoll > 0);
    itemList.nextItemButton:SetEnabled(#itemsToRoll > 0);
end

--Add array to frame function.
local function addArrayToFrame(frame)
	for _,v in ipairs(itemsToRoll) do 
		frame:AddMessage(v[1] .. " - " .. v[2], v[3], v[4], v[5]);
	end
end

--Function for updating the scroll list message frame.
local function updateMessageframe(frame)
    frame:Clear();
    
    if not itemsToRoll then
        return;
    end

	local arrLength = #itemsToRoll;

    if arrLength > 0 then
        frame:SetMaxLines(arrLength);
        
        table.sort(itemsToRoll, 
        function(a,b)
            if a[2] > b[2] then
                return false;
            elseif a[2] < b[2] then
                return true;
            else 
                return a[1] < b[1];
            end
        end);
        
        addArrayToFrame(frame);
        frame:SetScrollOffset(arrLength-21);
    end
    buttonControl();
end

--Function for registering an incomming item.
local function registerItem(sender, item)
    local _, itemLink = GetItemInfo(item);

    if itemLink and itemLink ~= "" then
        local _, playerClass = UnitClass(sender);
        local color = RAID_CLASS_COLORS[playerClass];
        
        functions.displayItemList();
        
        tinsert(itemsToRoll, 
        {
            sender, 
            itemLink, 
            color.r, 
            color.g, 
            color.b
        });

        updateMessageframe(itemList.itemFrame.messageFrame);
    end
end


-----------------------
-----EVENT HANDLING----
-----------------------


loadFrame:SetScript("OnEvent", 
function(self, event, ...)
	local arg1, arg2 = ...;
    if event == "CHAT_MSG_WHISPER" and UnitInRaid("player") and (UnitIsGroupLeader("player") or llrSettings.leadMode) then
        local name, _ = strsplit("-", arg2);

        if UnitPlayerOrPetInRaid(name) then
            registerItem(name, arg1);
        end
    end
end);


-----------------------
------ITEM WINDOW------
-----------------------


itemList:SetToplevel("true")
itemList:SetSize(200,235);
itemList:SetMovable(true);
itemList:EnableMouse(true);
itemList:RegisterForDrag("LeftButton");

itemList:SetScript("OnDragStart", itemList.StartMoving)
itemList:SetScript("OnDragStop", itemList.StopMovingOrSizing)
itemList:SetScript("OnShow", 
function(self) 
	functions.updateItemListScale();
end);

itemList.title = itemList:CreateFontString(nil,"OVERLAY");
itemList.title:SetFontObject("GameFontHighLight");
itemList.title:SetPoint("LEFT", itemList.TitleBg, "left", 5, -1);
itemList.title:SetText("LLR Item List");

itemList:SetClampedToScreen(true);
itemList:SetClampRectInsets(0, 0, 0, 0);
itemList:Hide();

--Next item button.
itemList.nextItemButton = CreateFrame("Button", nil, itemList, "GameMenuButtonTemplate");
itemList.nextItemButton:Disable();

functions.startButton(itemList.nextItemButton, itemList, "TOPRIGHT", "TOP", 0, -22, "Post Item", (itemList:GetWidth()-13)/2, 30, 
function(self) 
    if(#itemsToRoll > 0) then 
        local head = tremove(itemsToRoll, 1);

        SendChatMessage(head[1] .. " - " .. head[2], "RAID_WARNING", nil, nil);
        updateMessageframe(itemList.itemFrame.messageFrame);
    end
end);

--Skip item button
itemList.skipItemButton = CreateFrame("Button", nil, itemList, "GameMenuButtonTemplate");
itemList.skipItemButton:Disable();

functions.startButton(itemList.skipItemButton, itemList, "TOPLEFT", "TOP", 0, -22, "Skip Item", (itemList:GetWidth()-13)/2, 30, 
function(self) 
    if(#itemsToRoll > 0) then 
        tremove(itemsToRoll, 1);

        updateMessageframe(itemList.itemFrame.messageFrame);
    end
end);


--Initialize the frames for the rolled lists.
itemList.itemFrame  = CreateFrame("Frame", "ItemList", itemList);

--Start the item list frame
functions.scrollFrameFactory(itemList.itemFrame, itemList, 11, -35, "", itemList:GetWidth()-20, itemList:GetHeight()-20, false);

--Enable hyperlink functionality for the messageframe.
itemList.itemFrame.messageFrame:SetHyperlinksEnabled(1);

--Enable control click for items in the frame, in case leader wants to roll item from spesific location in the table.
itemList.itemFrame.messageFrame:SetScript("OnHyperlinkClick", 
    function (self, arg2, link)
        if IsControlKeyDown() then
            for i, v in ipairs(itemsToRoll) do
                if v[2] == link then
                    local head = tremove(itemsToRoll, i);
                    updateMessageframe(itemList.itemFrame.messageFrame);
                    SendChatMessage(head[1] .. " - " .. head[2], "RAID_WARNING", nil, nil);
                    break;
                end
            end
        end
    end
);

--Enable standard mouseover tooltip functionality.
itemList.itemFrame.messageFrame:SetScript("OnHyperlinkEnter", 
    function(_, _, link)
        GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR"); 
		GameTooltip:ClearLines(); 
        GameTooltip:SetHyperlink(link); 
        GameTooltip:AddLine("|cFFFFFF00CTRL click to post this item in a raid warning.")
		GameTooltip:Show();
    end
);

itemList.itemFrame.messageFrame:SetScript("OnHyperlinkLeave", functions.hideGametoolTip);


-----------------------
---GLOBAL FUNCTIONS----
-----------------------


--Function for updating the user window scale.
function functions.updateItemListScale()
	itemList:SetScale(variables.screenScale*llrSettings.itemListScale);
end

--Function for displaying the user window.
function functions.displayItemList()
	if not itemList:IsShown() then
		itemList:Show();
	end 
end

--Function for hiding the user window.
function functions.hideItemList()
	if itemList:IsShown() then
		itemList:Hide();
	end 
end

--Function to determine if the user window is displayed.
function functions.isItemListShown()
	return itemList:IsShown();
end

--Function for resetting the user window position.
function functions.resetItemListWindow()
	itemList:ClearAllPoints();
	itemList:SetPoint("CENTER", UIParent, "CENTER");
end

--Function to toggle the whisper listener.
function functions.toggleWhisperListener()
    if llrSettings.whisperListener and UnitInRaid("player") then
        loadFrame:RegisterEvent("CHAT_MSG_WHISPER");
    else 
        loadFrame:UnregisterEvent("CHAT_MSG_WHISPER");
    end
end